@@ -48,6 +48,7 @@ items_in_order = [
     "simulated_python_traceback",  # <-- python_tb()
     "shortened_traceback",  # <-- friendly_tb()
     "suggest",  # <-- hint()
+    "warning message",
     "generic",  # <-- what()
     "parsing_error",
     "parsing_error_source",
@@ -58,6 +59,10 @@ items_in_order = [
     "exception_raised_header",
     "exception_raised_source",
     "exception_raised_variables",
+    "warning location header",
+    "warning source",
+    "warning variables",
+    "additional variable warning",
 ]
 
 
@@ -68,6 +73,7 @@ repl_indentation = {
     "original_python_traceback": "none",
     "shortened_traceback": "none",
     "suggest": "double",
+    "warning message": "single",
     "generic": "single",
     "parsing_error": "single",
     "parsing_error_source": "none",
@@ -78,6 +84,10 @@ repl_indentation = {
     "exception_raised_header": "single",
     "exception_raised_source": "none",
     "exception_raised_variables": "double",
+    "warning location header": "single",
+    "warning source": "none",
+    "warning variables": "double",
+    "additional variable warning": "single",
 }
 
 
@@ -89,15 +99,19 @@ def repl(info: Info, include: InclusionChoice = "friendly_tb") -> str:
     """
     if include == "message":
         return info["message"]
-    elif include == "detailed_tb":
-        return detailed_tb(info)
+    if include == "detailed_tb":
+        if "detailed_tb" in info:
+            return detailed_tb(info)
+        else:
+            return ""
     items_to_show = select_items(include)
     spacing = {"single": " " * 4, "double": " " * 8, "none": ""}
     result = [""]
     for item in items_to_show:
         if item in info:
             indentation = spacing[repl_indentation[item]]
-            for line in info[item].split("\n"):
+            lines = info[item].split("\n")
+            for line in lines:
                 result.append(indentation + line)
 
     if result == [""] or not result:
@@ -107,7 +121,12 @@ def repl(info: Info, include: InclusionChoice = "friendly_tb") -> str:
 
 
 def detailed_tb(info: Info) -> str:  # Special case
-    # TODO: document this
+    """Unlike the normal information from 'where()', which focus on at
+    most two frames, detailed_tb() gives information for all the frames.
+    It is used mostly in IPython based environment - especially with
+    the 'button-based' mode in Jupyter notebooks/lab."""
+    if "detailed_tb" not in info:
+        return ""
     result = [""]
     spacing = {"location": " " * 4, "var_info": " " * 8}
     for location, source, var_info in info["detailed_tb"]:
@@ -178,7 +197,7 @@ items_groups: Dict[InclusionChoice, Set[str]] = {
     "message": {"message"},  # Also included as last line of traceback
     "hint": {"suggest"},
     "what": {"generic"},
-    "why": {"cause"},
+    "why": {"warnings", "cause"},
     "where": {
         "parsing_error",
         "parsing_error_source",
@@ -188,8 +207,12 @@ items_groups: Dict[InclusionChoice, Set[str]] = {
         "exception_raised_header",
         "exception_raised_source",
         "exception_raised_variables",
+        "warning location header",
+        "warning source",
+        "warning variables",
+        "additional variable warning",
     },
-    "friendly_tb": {"shortened_traceback", "suggest"},
+    "friendly_tb": {"shortened_traceback", "suggest", "warnings", "warning message"},
     "python_tb": {"simulated_python_traceback"},
     "debug_tb": {"original_python_traceback"},
     "detailed_tb": {"detailed_tb"},
