@@ -29,25 +29,6 @@ def _nothing_to_show():
     return _("Nothing to show: no exception recorded.")
 
 
-def back() -> None:
-    """Removes the last recorded traceback item.
-
-    The intention is to allow recovering from a typo when trying interactively
-    to find out specific information about a given exception.
-    """
-    if not session.recorded_tracebacks:
-        info = {"message": _("Nothing to go back to: no exception recorded.") + "\n"}
-        explanation = session.formatter(info, include="message")
-        session.write_err(explanation)
-        return
-    session.recorded_tracebacks.pop()
-    if session.recorded_tracebacks:
-        info = session.recorded_tracebacks[-1].info
-        if info["lang"] != friendly_traceback.get_lang():
-            info["lang"] = friendly_traceback.get_lang()
-            session.recorded_tracebacks[-1].recompile_info()
-
-
 def disable():
     """Disable friendly-traceback exception hook"""
     if not session.installed:
@@ -91,18 +72,44 @@ def hint(index: int = -1) -> None:
     explain(index, include="hint")
 
 
-def history() -> None:
-    """Prints the list of error messages recorded so far."""
-    if not session.recorded_tracebacks:
-        info = {"message": _nothing_to_show() + "\n"}
-        explanation = session.formatter(info, include="message")
-        session.write_err(explanation)
-        return
-    for index, tb in enumerate(session.recorded_tracebacks):
-        if "message" in tb.info:
-            info = {"message": f"{index+1}. {tb.info['message']}"}
+class History:
+    __name__ = "history"
+
+    def __delitem__(self, index):
+        """Used to delete a particular item: del history[index]"""
+        if not session.recorded_tracebacks:
+            info = {"message": _("Nothing to delete: no exception recorded.") + "\n"}
             explanation = session.formatter(info, include="message")
             session.write_err(explanation)
+            return
+        del session.recorded_tracebacks[index]
+
+    def __repr__(self):
+        return (
+            _("Shows a list of recorded traceback messages.")
+            + "\n"
+            + _("You can also use `history.clear()` and `del history[index]`.")
+        )
+
+    def clear(self):
+        """Removes all recorded tracebacks and warnings"""
+        session.recorded_tracebacks.clear()
+
+    def __call__(self):
+        """Prints a list of recorded tracebacks and warning messages"""
+        if not session.recorded_tracebacks:
+            info = {"message": _nothing_to_show() + "\n"}
+            explanation = session.formatter(info, include="message")
+            session.write_err(explanation)
+            return
+        for index, tb in enumerate(session.recorded_tracebacks):
+            if "message" in tb.info:
+                info = {"message": f"{index}. {tb.info['message']}"}
+                explanation = session.formatter(info, include="message")
+                session.write_err(explanation)
+
+
+history = History()
 
 
 def python_tb(index: int = -1) -> None:
@@ -307,7 +314,10 @@ def _get_exception() -> Optional[BaseException]:  # pragma: no cover
     if not session.recorded_tracebacks:
         print(_nothing_to_show())
         return None  # add explicit None here and elsewhere to silence mypy
-    return session.recorded_tracebacks[-1].tb_data.exception_instance
+    current_tb = session.recorded_tracebacks[-1]
+    if hasattr(current_tb, "tb_data"):
+        return current_tb.tb_data.exception_instance
+    return current_tb.warning_instance
 
 
 def _get_frame() -> Optional[types.FrameType]:  # pragma: no cover
@@ -318,7 +328,10 @@ def _get_frame() -> Optional[types.FrameType]:  # pragma: no cover
     if not session.recorded_tracebacks:
         print(_nothing_to_show())
         return None
-    return session.recorded_tracebacks[-1].tb_data.exception_frame
+    current_tb = session.recorded_tracebacks[-1]
+    if hasattr(current_tb, "tb_data"):
+        return current_tb.tb_data.exception_frame
+    return current_tb.frame  # warnings
 
 
 def _get_statement() -> Optional[Statement]:  # pragma: no cover
@@ -331,11 +344,15 @@ def _get_statement() -> Optional[Statement]:  # pragma: no cover
     if not session.recorded_tracebacks:
         print(_nothing_to_show())
         return None
-    if isinstance(
-        session.recorded_tracebacks[-1].tb_data.exception_instance, SyntaxError
-    ):
-        return session.recorded_tracebacks[-1].tb_data.statement
-    print("No statement: not a SyntaxError.")
+    current_tb = session.recorded_tracebacks[-1]
+    if hasattr(current_tb, "tb_data"):
+        if isinstance(
+            session.recorded_tracebacks[-1].tb_data.exception_instance, SyntaxError
+        ):
+            return session.recorded_tracebacks[-1].tb_data.statement
+    elif hasattr(current_tb, "problem_statement"):
+        return current_tb.problem_statement
+    print("No statement: likely not a SyntaxError nor a SyntaxWarning.")
     return None
 
 
@@ -348,7 +365,10 @@ def _get_tb_data() -> Optional[TracebackData]:  # pragma: no cover
     if not session.recorded_tracebacks:
         print(_nothing_to_show())
         return None
-    return session.recorded_tracebacks[-1].tb_data
+    current_tb = session.recorded_tracebacks[-1]
+    if hasattr(current_tb, "tb_data"):
+        return current_tb.tb_data
+    return current_tb
 
 
 def _get_info() -> list:
@@ -386,7 +406,6 @@ helpers: Dict[str, Callable[..., None]] = {
     "hint": hint,
     "enable": enable,
     "disable": disable,
-    "back": back,
     "history": history,
     "friendly_tb": friendly_tb,
     "python_tb": python_tb,
