@@ -7,22 +7,18 @@ they are considered to be internal functions, subject to change at any
 time. If functions defined in friendly_traceback.__init__.py do not meet your needs,
 please file an issue.
 """
-import contextlib
-import inspect
-import io
+
 import re
-import sys
 import traceback
 import types
-from itertools import dropwhile
-from typing import List, Optional, Sequence, Tuple, Type
+from typing import List, Optional, Sequence, Type
 
-from . import debug_helper, info_generic, info_specific, info_variables
+from . import debug_helper, info_generic, info_variables, message_parser, tb_data
 from .frame_info import FrameInfo
 from .ft_gettext import current_lang
-from .path_info import is_excluded_file, path_utils
+from .path_info import path_utils
 from .source_cache import cache
-from .syntax_errors import analyze_syntax, indentation_error, source_info
+from .syntax_errors import analyze_syntax, indentation_error
 from .typing_info import _E, Info
 
 try:
@@ -30,264 +26,10 @@ try:
 except ImportError:  # pragma: no cover
     pass  # ignore errors when processed by Sphinx
 
+TracebackData = tb_data.TracebackData
 
-STR_FAILED = "<exception str() failed>"  # Same as Python
 _ = current_lang.translate
 
-
-def convert_value_to_message(value: BaseException) -> str:
-    """This converts the 'value' of an exception into a string, while
-    being safe to use for custom exceptions which have been incorrectly
-    defined. See https://github.com/aroberge/friendly/issues/181 for an example.
-    """
-    try:
-        message = str(value)
-    except Exception:  # noqa
-        message = STR_FAILED
-    return message
-
-
-def retrieve_message(etype: Type[_E], value: _E, tb: types.TracebackType) -> str:
-    "Safely retrieves the message, including any additional hint from Python."
-    message = convert_value_to_message(value)
-    if (
-        message == STR_FAILED
-        or sys.version_info < (3, 10)
-        or etype not in (AttributeError, NameError)
-    ):
-        return message
-    # 3.10+ hints are not directly accessible from Python.
-    err = io.StringIO()
-    with contextlib.redirect_stderr(err):
-        sys.__excepthook__(etype, value, tb)
-    full_message = err.getvalue().split("\n")[-2]
-    return full_message.split(":", 1)[1].strip()
-
-
-class TracebackData:
-    """Raw traceback info obtained from Python.
-
-    Instances of this class are intended to include all the relevant
-    information about an exception so that a FriendlyTraceback object
-    can be created.
-    """
-
-    def __init__(self, etype: Type[_E], value: _E, tb: types.TracebackType) -> None:
-        """This object is initialized with the standard values for a
-        traceback::
-
-            etype, value, tb = sys.exc_info()
-        """
-        cache.remove("<fstring>")
-        self.exception_type = etype
-        self.exception_name = etype.__name__
-        self.value = value
-        self.message = retrieve_message(etype, value, tb)
-        self.formatted_tb = traceback.format_exception(etype, value, tb)
-        self.records = self.get_records(tb)
-        self.python_records = self.get_records(tb, python_excluded=False)
-
-        # The following three attributes get their correct values in get_source_info()
-        self.bad_line = "\n"
-        self.filename = ""
-        self.exception_frame = None
-        self.program_stopped_frame = None
-        self.program_stopped_bad_line = "\n"
-        self.get_source_info()
-
-        # The following attributes get their correct values in self.locate_error()
-        self.node = None
-        self.node_text = ""
-        self.node_range: Optional[Tuple[int, int]] = None
-        self.original_bad_line = self.bad_line
-        self.program_stopped_node_range = None
-
-        if issubclass(etype, SyntaxError):
-            self.statement: Optional[source_info.Statement] = source_info.Statement(
-                self.value, self.bad_line
-            )
-            # Removing extra ending spaces for potentially shorter displays later on
-
-            def remove_space(text: str) -> str:
-                if text.rstrip():
-                    if text.endswith("\n"):
-                        return text.rstrip() + "\n"
-                    return text.rstrip()
-                return text
-
-            self.statement.entire_statement = remove_space(
-                self.statement.entire_statement
-            )
-            self.statement.bad_line = remove_space(self.statement.bad_line)
-        else:
-            self.statement = None
-            self.locate_error()
-
-    def get_records(
-        self, tb: types.TracebackType, python_excluded: bool = True
-    ) -> List[FrameInfo]:
-        """Get the traceback frame history, excluding those originating
-        from our own code that are included either at the beginning or
-        at the end of the traceback.
-        """
-        try:
-            all_records = list(FrameInfo.stack_data(tb, collapse_repeated_frames=False))
-            records = list(
-                dropwhile(
-                    lambda record: is_excluded_file(
-                        record.filename, python_excluded=python_excluded
-                    ),
-                    all_records,
-                )
-            )
-            records.reverse()
-            records = list(
-                dropwhile(
-                    lambda record: is_excluded_file(
-                        record.filename, python_excluded=python_excluded
-                    ),
-                    records,
-                )
-            )
-            records.reverse()
-            if records or issubclass(self.exception_type, (SyntaxError, MemoryError)):
-                return records
-        except AssertionError:  # from stack_data
-            # problems may arise when SyntaxErrors are raise
-            # from a normal console like the one used in Mu.
-            all_records = inspect.getinnerframes(tb, cache.context)
-            records = list(
-                dropwhile(
-                    lambda record: is_excluded_file(
-                        record.filename, python_excluded=python_excluded
-                    ),
-                    all_records,
-                )
-            )
-            records.reverse()
-            records = list(
-                dropwhile(
-                    lambda record: is_excluded_file(
-                        record.filename, python_excluded=python_excluded
-                    ),
-                    records,
-                )
-            )
-            records.reverse()
-            if records or issubclass(self.exception_type, (SyntaxError, MemoryError)):
-                return records
-        # If all the records are removed, it likely means that all the error
-        # is in our own code - or that of the user who chose to exclude
-        # some files. If so, we make sure to have something to analyze
-        # and help identify the problem.
-        return all_records  # pragma: no cover
-
-    def get_source_info(self) -> None:
-        """Retrieves the file name and the line of code where the exception
-        was raised.
-        """
-        if issubclass(self.exception_type, SyntaxError):
-            self.filename = self.value.filename
-            # Python 3.10 introduced new arguments. For simplicity,
-            # we give them some default values for other Python versions
-            # so that we can use these elsewhere without having to perform
-            # additional checks.
-            if not hasattr(self.value, "end_offset"):
-                self.value.end_offset = (
-                    self.value.offset + 1 if self.value.offset else 0
-                )
-                self.value.end_lineno = self.value.lineno
-
-            # Normally, when an error occurs entirely on a given line,
-            # the end offset should be at least one more than the offset.
-            # However, as noted in issue #34, that might not always be the case.
-            # To show the location of the error, we do need to have
-            # offset and end_offset be different.
-            if (
-                self.value.end_lineno == self.value.lineno
-                and self.value.end_offset == self.value.offset
-            ):
-                self.value.end_offset += 1
-
-            if self.value.text is not None:
-                self.bad_line = self.value.text  # typically includes "\n"
-                return
-
-            # this can happen with editors_helpers.check_syntax()
-            try:
-                self.bad_line = cache.get_source_lines(self.filename)[
-                    self.value.lineno - 1
-                ]
-            except Exception:  # noqa
-                self.bad_line = "\n"
-            return
-
-        if self.records:
-            record = self.records[-1]
-            self.exception_frame = record.frame
-            self.filename = record.filename
-            line = record.problem_line()
-            self.bad_line = line.strip()  # fix for 3.11
-            # protecting against https://github.com/alexmojaki/stack_data/issues/13
-            if not self.bad_line:
-                try:
-                    lines = cache.get_source_lines(record.filename)
-                    self.bad_line = lines[record.lineno - 1]
-                except Exception:  # noqa
-                    debug_helper.log("Could not get bad_line")
-
-            if len(self.records) > 1:
-                record = self.records[0]
-                line = record.problem_line()
-                self.program_stopped_frame = record.frame
-                self.program_stopped_bad_line = line.rstrip()
-            else:
-                self.program_stopped_bad_line = self.bad_line
-                self.program_stopped_frame = self.exception_frame
-            return
-
-        if issubclass(self.exception_type, MemoryError):
-            self.bad_line = "<not available>"
-            return
-
-        # We should never reach this stage.
-        def _log_error() -> None:  # pragma: no cover
-            debug_helper.log("Internal error in TracebackData.get_source_info.")
-            debug_helper.log("No records found.")
-            debug_helper.log("self.exception_type:" + str(self.exception_type))
-            debug_helper.log("self.value:" + str(self.value))
-            debug_helper.log_error()
-
-        _log_error()  # pragma: no cover
-
-    def locate_error(self) -> None:
-        """Attempts to narrow down the location of the error so that,
-        if possible, the problem code is highlighted with ^^^^."""
-        if not self.records:  # pragma: no cover
-            if issubclass(self.exception_type, MemoryError):
-                return
-            debug_helper.log("No records in locate_error().")
-            return
-
-        node_info = self.records[-1].node_info  # noqa
-        if node_info:
-            self.node, _ignore, self.node_text = node_info
-            if self.node_text.strip():
-                # Replacing the line that caused the exception by the text
-                # of the 'node' facilitates the process of identifying the cause.
-                # However, in a few cases, we do need to keep the entire original line.
-                self.original_bad_line = self.bad_line
-                self.bad_line = self.node_text
-
-        # Also attempt to restrict the information about where the program
-        # stopped to the strict minimum so that we don't show irrelevant
-        # values of names
-        if self.records[0].node_info and self.records[0].node_info != node_info:
-            node, _ignore, node_text = self.records[0].node_info
-            if node_text.strip():
-                self.program_stopped_bad_line = node_text
-
-
 # ====================
 # The following is an example of a formatted traceback, with
 # some parts identified with (partial) names used below
@@ -369,23 +111,27 @@ class FriendlyTraceback:
             etype, value, tb = sys.exc_info()
 
         The "header" key for the info dict is assigned here."""
+        from .config import session
+
         try:
-            self.tb_data = TracebackData(etype, value, tb)
+            self.tb_data = tb_data.TracebackData(etype, value, tb)
         except Exception as e:  # pragma: no cover
             debug_helper.log("Uncaught exception in TracebackData:")
+            if debug_helper.DEBUG:
+                raise
             debug_helper.handle_internal_error(e)
             raise SystemExit
+        self.tb = tb
         self.suppressed = ["       ... " + _("More lines not shown.") + " ..."]
-        self.info = {"header": _("Python exception:")}
-        self.message = self.assign_message(etype, value, tb)  # language independent
+        self.info = {"header": _("Python exception:")}  # Used by HackInScience
+        self.info["lang"] = session.lang
+        self.message = self.assign_message(etype, value)  # language independent
         self.assign_tracebacks()
 
         # include some values for debugging purpose in an interactive session
-        self.info["_exc_instance"] = value
-        self.info["_frame"] = self.tb_data.exception_frame
-        self.info["_tb_data"] = self.tb_data
+        self.tb_data.exception_instance = value
 
-    def assign_message(self, etype, value, tb) -> str:
+    def assign_message(self, etype, value) -> str:
         """Assigns the error message, as the attribute ``message``
         which is something like::
 
@@ -393,10 +139,10 @@ class FriendlyTraceback:
         """
         exc_name = etype.__name__
         if hasattr(value, "msg"):
-            self.info["message"] = f"{exc_name}: {value.msg}\n"
+            message = value.msg
         else:
-            message = retrieve_message(etype, value, tb)
-            self.info["message"] = f"{exc_name}: {message}\n"
+            message = self.tb_data.full_message
+        self.info["message"] = f"{exc_name}: {message}\n"
         return self.info["message"]
 
     def compile_info(self) -> None:
@@ -415,7 +161,10 @@ class FriendlyTraceback:
         """This is useful if we need to redisplay some information in a
         different language than what was originally used.
         """
+        from .config import session
+
         self.info["header"] = _("Python exception:")
+        self.info["lang"] = session.lang
         self.assign_tracebacks()
         self.compile_info()
 
@@ -433,7 +182,7 @@ class FriendlyTraceback:
         except Exception:
             return
 
-        if STR_FAILED in self.message:
+        if tb_data.STR_FAILED in self.message:
             self.info["cause"] = _(
                 "Warning: improperly formed exception.\n"
                 "I suspect that a custom exception has been raised\n"
@@ -460,11 +209,9 @@ class FriendlyTraceback:
         the latter being the "hint" appended to the friendly traceback.
         """
         etype = self.tb_data.exception_type
-        value = self.tb_data.value
+        message = self.tb_data.message
 
-        cause = info_specific.get_likely_cause(
-            etype, value, self.tb_data.exception_frame, self.tb_data
-        )  # [3]
+        cause = message_parser.get_likely_cause(etype, message, self.tb_data)  # [3]
         self.info.update(**cause)
 
     def set_cause_syntax(self) -> None:
@@ -538,10 +285,12 @@ class FriendlyTraceback:
             return
 
         self.locate_exception_raised(records[-1])
+        if len(records) < 2:
+            return
+
         self.info["detailed_tb"] = self.get_detailed_stack_info(records)
-        if len(records) > 1:
-            _ignore, partial_source, var_info = self.info["detailed_tb"][0]
-            self.locate_last_call(records[0], partial_source, var_info)
+        _ignore, partial_source, var_info = self.info["detailed_tb"][0]
+        self.locate_last_call(records[0], partial_source, var_info)
 
     def locate_exception_raised(self, record: FrameInfo) -> None:
         """Sets the values of the following attributes which are
@@ -558,9 +307,9 @@ class FriendlyTraceback:
             self.info["exception_raised_header"] = ""
             return
 
-        partial_source = record.partial_source_with_node_range
-        if partial_source["source"].strip() == "0:":
-            partial_source["source"] = ""
+        source = record.partial_source_with_node_range
+        if source.strip() == "0:":
+            source = ""
         filename = path_utils.shorten_path(record.filename)
 
         unavailable = filename in ["<unknown>", "<string>"]
@@ -571,18 +320,17 @@ class FriendlyTraceback:
 
         if session.ipython_prompt and filename.startswith("["):
             self.info["exception_raised_header"] = _(
-                "Exception raised on line {linenumber} of code block {filename}.\n"
+                "Exception raised on line `{linenumber}` of code block {filename}.\n"
             ).format(linenumber=record.lineno, filename=filename)
         else:
             self.info["exception_raised_header"] = _(
-                "Exception raised on line {linenumber} of file {filename}.\n"
+                "Exception raised on line `{linenumber}` of file '{filename}'.\n"
             ).format(linenumber=record.lineno, filename=filename)
 
         if unavailable:
             return
 
-        source = partial_source["source"]
-        if source.strip() in ["-->1:", "1:"]:
+        if source.strip() == "<NO SOURCE>":
             source = _(
                 "{filename} is not a regular Python file whose contents can be analyzed.\n"
             ).format(filename=filename)
@@ -591,12 +339,15 @@ class FriendlyTraceback:
 
         if self.tb_data.node_text:
             line = self.tb_data.node_text
-        else:
-            line = partial_source["line"]
+        else:  # This almost never happens
+            line = record.problem_line()
 
         var_info = info_variables.get_var_info(line, record.frame)
-        if var_info:
-            self.info["exception_raised_variables"] = var_info
+        self.info["exception_raised_variables"] = var_info["var_info"]
+        if "additional variable warning" in var_info:
+            self.info["additional variable warning"] = var_info[
+                "additional variable warning"
+            ]
 
     def locate_last_call(self, record: FrameInfo, partial_source, var_info) -> None:
         """Sets the values of the following attributes:
@@ -605,23 +356,18 @@ class FriendlyTraceback:
         * exception_raised_source
         * last_call_variables
         """
-        # partial_source = record.partial_source_with_node_range
         filename = path_utils.shorten_path(record.filename)
 
         if filename and "[" in filename:
             self.info["last_call_header"] = _(
-                "Execution stopped on line {linenumber} of code block {filename}.\n"
+                "Execution stopped on line `{linenumber}` of code block {filename}.\n"
             ).format(linenumber=record.lineno, filename=filename)
         else:
             self.info["last_call_header"] = _(
-                "Execution stopped on line {linenumber} of file {filename}.\n"
+                "Execution stopped on line `{linenumber}` of file '{filename}'.\n"
             ).format(linenumber=record.lineno, filename=filename)
-        # self.info["last_call_source"] = partial_source["source"]
         self.info["last_call_source"] = partial_source
 
-        # var_info = info_variables.get_var_info(
-        #     self.tb_data.program_stopped_bad_line, record.frame
-        # )
         if var_info:
             self.info["last_call_variables"] = var_info
 
@@ -641,14 +387,14 @@ class FriendlyTraceback:
             partial_source = record.partial_source_with_node_range
             var_info = info_variables.get_var_info(line, record.frame)
             if "[" in filename:
-                location = _("Code block {filename}, line {line}").format(
+                location = _("Code block {filename}, line `{line}`").format(
                     filename=filename, line=lineno
                 )
             else:
-                location = _("File {filename}, line {line}").format(
+                location = _("File '{filename}', line `{line}`").format(
                     filename=filename, line=lineno
                 )
-            detailed_tb.append((location, partial_source["source"], var_info))
+            detailed_tb.append((location, partial_source, var_info["var_info"]))
         return detailed_tb
 
     def locate_parsing_error(self) -> None:
@@ -688,7 +434,7 @@ class FriendlyTraceback:
             if filepath in ["<string>", "<stdin>"] and self.tb_data.value.lineno != 1:
                 self.info["parsing_error"] += not_regular_file
         elif filepath:  # could be None
-            self.info["parsing_error"] = could_not_understand + ".\n"
+            self.info["parsing_error"] = could_not_understand + "\n"
 
         self.info["parsing_error_source"] = f"{partial_source}\n"
 
@@ -713,8 +459,12 @@ class FriendlyTraceback:
         * simulated_python_traceback
         * shortened_traceback
         """
+        from .config import session
+
         if not hasattr(self, "message"):
-            self.assign_message()
+            self.assign_message(
+                self.tb_data.exception_type, self.tb_data.value, self.tb
+            )
         if isinstance(self.tb_data.formatted_tb, str):
             # for example: "Traceback not available from IDLE" ...
             tb = self.info["message"]
@@ -750,9 +500,9 @@ class FriendlyTraceback:
 
         if "RecursionError" in full_tb[-1]:
             if len(shortened_tb) > 12:
-                shortened_tb = shortened_tb[0:5] + self.suppressed + shortened_tb[-5:]
+                shortened_tb = shortened_tb[:5] + self.suppressed + shortened_tb[-5:]
             if len(python_tb) > 12:
-                python_tb = python_tb[0:5] + self.suppressed + python_tb[-5:]
+                python_tb = python_tb[:5] + self.suppressed + python_tb[-5:]
 
         exc = self.tb_data.value
         chain_info = ""
@@ -770,15 +520,19 @@ class FriendlyTraceback:
         self.info["simulated_python_traceback"] = (
             chain_info + "\n".join(python_tb) + "\n"
         )
-        self.info["shortened_traceback"] = (
-            short_chain_info + "\n".join(shortened_tb) + "\n"
-        )
         self.info["original_python_traceback"] = chain_info + "\n".join(full_tb) + "\n"
         # The following is needed for some determining the cause in at
         # least one case.
         # skipcq: PYL-W0201
         self.tb_data.simulated_python_traceback = "\n".join(python_tb) + "\n"
 
+        if session.include_chained_exception:
+            self.info["shortened_traceback"] = (
+                short_chain_info + "\n".join(shortened_tb) + "\n"
+            )
+        else:
+            self.info["shortened_traceback"] = "\n".join(shortened_tb) + "\n"
+
     def shorten(self, tb: Sequence[str]) -> List[str]:
         """Shortens a traceback (as list of lines)
         by removing lines if it exceeds a certain length
@@ -791,7 +545,7 @@ class FriendlyTraceback:
         for line in shortened_tb:
             match = re.search(pattern, line)
             if match:
-                filename = match.group(1)
+                filename = match[1]
                 short_filename = path_utils.shorten_path(filename)
                 line = line.replace(filename, short_filename)
                 if (
@@ -801,6 +555,8 @@ class FriendlyTraceback:
                 ):
                     line = line.replace("  File", "  Code block")
                     line = line.replace('"[', "[").replace(']"', "]")
+                    parts = line.split(",")
+                    line = ",".join(parts[:2])
             temp.append(line)
         return temp
 
@@ -811,22 +567,18 @@ class FriendlyTraceback:
         """
         result = []
         for record in records:
-            partial_source = record.partial_source  # noqa
             result.append(
-                '  File "{}", line {}, in {}'.format(
-                    record.filename, record.lineno, record.code.co_name
-                )
+                f'  File "{record.filename}", line {record.lineno}, in {record.code.co_name}'
             )
-            bad_line = partial_source["line"]
-            if bad_line is not None:
-                result.append("    {}".format(bad_line.strip()))
+            bad_line = record.problem_line()
+            result.append(f"    {bad_line.strip()}")
 
         if issubclass(self.tb_data.exception_type, SyntaxError):
             value = self.tb_data.value
             offset = value.offset
             filename = value.filename
             lines = cache.get_source_lines(filename)
-            result.append('  File "{}", line {}'.format(filename, value.lineno))
+            result.append(f'  File "{filename}", line {value.lineno}')
             _line = value.text
             if _line is None:
                 try:
@@ -848,14 +600,13 @@ class FriendlyTraceback:
                     if (
                         value.end_lineno is not None
                         and value.end_lineno != value.lineno
-                        or value.end_offset is not None
-                        and value.end_offset < 1
                     ):
                         nb_carets = len(bad_line) - offset + 1
                         continuation = "-->"
                     else:
                         nb_carets = value.end_offset - offset if value.end_offset else 1
                         continuation = ""
+                    nb_carets = max(nb_carets, 1)
                     offset = offset - (len(_line) - len(bad_line))  # removing indent
                     # In some IndentationError cases, and possibly others,
                     # Python's computed offset would show the ^ just before the first token
@@ -863,7 +614,7 @@ class FriendlyTraceback:
                         offset = 1
                         nb_carets = 1
                         continuation = ""
-                    result.append("    {}".format(bad_line))
+                    result.append(f"    {bad_line}")
                     result.append(" " * (3 + offset) + "^" * nb_carets + continuation)
         result.append(self.info["message"].strip())
         return result
