@@ -68,8 +68,9 @@ class Cache:
             lines = self.local_cache[filename]
         if not lines:  # can happen for f-strings and frozen modules
             lines = []
-        lines.append("\n")  # required when dealing with EOF errors
-        return lines
+        # Adding ["\n"] is required when dealing with EOF errors
+        # Do not use append; see #174.
+        return lines + ["\n"]
 
 
 cache = Cache()
