@@ -419,6 +419,12 @@ def assign_to_literal(message: str = "", statement=None):
         ).format(literal=literal, name=name, of_type=of_type)
         + suggest
     )
+    if statement.prev_token == "{":
+        cause += "\n" + _(
+            "It is possible that "
+            "you used an equal sign `=` instead of a colon `:`\n"
+            "to assign values to keys in a dict.\n"
+        )
     return {"cause": cause, "suggest": hint}
 
 
@@ -532,7 +538,7 @@ def annotated_name_cannot_be_global(message: str = "", statement=None):
     cause = _(
         "The object named `{name}` is defined with type annotation\n"
         "as a local variable. It cannot be declared to be a global variable.\n"
-    ).format(name=match.group(1))
+    ).format(name=match[1])
     # Ensure that only the variable gets highlighted for consistency
     # across all Python versions.
     statement.location_markers = su.highlight_single_token(statement.bad_token)
@@ -571,10 +577,10 @@ def bracket_was_expected(message: str = "", statement=None):
         return {}
 
     cause = _("Python tells us that the {bracket} was never closed.\n").format(
-        bracket=su.name_bracket(match.group(1))
+        bracket=su.name_bracket(match[1])
     )
     hint = _("The {bracket} was never closed.\n").format(
-        bracket=su.name_bracket(match.group(1))
+        bracket=su.name_bracket(match[1])
     )
     rephrased_cause = statement_analyzer.unclosed_bracket(statement)
     if rephrased_cause:
@@ -878,6 +884,12 @@ def eol_while_scanning_string_literal(message: str = "", statement=None):
         "You started writing a string with a single or double quote\n"
         "but never ended the string with another quote on that line.\n"
     )
+
+    for token in statement.tokens:
+        if token.is_error():
+            statement.location_markers = su.highlight_single_token(token)
+            break
+
     # skipcq: PYL-R1714
     # second if case for Python 3.10
     if statement.prev_token == "\\" or statement.bad_line[-2] == "\\":
@@ -1078,7 +1090,7 @@ def from__future__not_defined(message: str = "", _statement=None):
     available = _("The available features are `{names}`.\n").format(
         names=utils.list_to_string(names).replace(",", ",\n")
     )
-    feature = match.group(1)
+    feature = match[1]
     if feature == "*":
         cause = _(
             "When using a `from __future__ import` statement,\n"
@@ -1366,10 +1378,10 @@ def mismatched_parenthesis(message: str = "", statement=None):
         if match is None:
             return {}
     else:
-        lineno = match.group(3)
+        lineno = match[3]
 
-    opening = match.group(2)
-    closing = match.group(1)
+    opening = match[2]
+    closing = match[1]
 
     cause = statement_analyzer.mismatched_brackets(statement)
     if cause:
@@ -1934,3 +1946,35 @@ def assign_to_others(message: str = "", _statement=None):
         return {}
     hint = _assign_to_identifiers_only()
     return {"cause": hint, "suggest": hint}
+
+
+@add_python_message
+def arguments_cannot_follow_var(message: str = "", statement=None):
+    # new message for Python 3.11
+    if "arguments cannot follow var-keyword argument" not in message:
+        return {}
+    return error_in_def.analyze_def_statement(statement)
+
+
+@add_python_message
+def slash_must_be_ahead(message: str = "", statement=None):
+    # new message for Python 3.11
+    if "/ must be ahead of *" not in message:
+        return {}
+    return error_in_def.analyze_def_statement(statement)
+
+
+@add_python_message
+def star_may_appear_only_once(message: str = "", statement=None):
+    # new message for Python 3.11
+    if "* argument may appear only once" not in message:
+        return {}
+    return error_in_def.analyze_def_statement(statement)
+
+
+@add_python_message
+def slash_must_appear_only_once(message: str = "", statement=None):
+    # new message for Python 3.11
+    if "/ may appear only once" not in message:
+        return {}
+    return error_in_def.analyze_def_statement(statement)
