@@ -162,6 +162,13 @@ def copy_pasted_code(statement):
     if first_token not in [">>", "..."]:
         return {}
 
+    if statement.bad_line.strip() == "....":
+        hint = _("Did you mean to write `...`?\n")
+        cause = _(
+            "It looks like you meant to write `...` but added an extra `.` by mistake.\n"
+        )
+        return {"cause": cause, "suggest": hint}
+
     hint = _("Did you use copy-paste?\n")
     if (
         first_token == ">>"
@@ -835,7 +842,7 @@ def invalid_octal(statement):
         "It looks like you used an invalid character (`{character}`) in an octal number.\n\n"
         "Octal numbers are base 8 integers that only use the symbols `0` to `7`\n"
         "to represent values.\n"
-        "In Python, hexadecimal numbers start with either `0o` or `0O`,\n"
+        "In Python, octal numbers start with either `0o` or `0O`,\n"
         "(the digit zero followed by the letter `o`)\n"
         "followed by the characters used to represent the value of that integer.\n"
     ).format(character=wrong.string[0])
